'====================================================================================================================
'Public Function CheckOnlineUpgrade(UpgradeSite as String) as Boolean
'
' If this returns true immediately "End" current App.
'
' UpgradeSite must evaluate to a URL which points to a web page with version information
'
' XML Format
' ----------
'     title   = The title of the VB application as determined by App.Title
'     version = The current version of the application in the form: major.minor.revision
'               The version should be obtained using CStr(App.Major) & "." & CStr(App.Minor) & "." & CStr(App.Revision)
'               Changes in the Major portion of the version will require an upgrade
'               Changes in the Minor portion of the version will prompt for an optional upgrade
'               Changes in the Revision will not prompt the user to upgrade at all
'     url     = The full URL to a executable upgrade routine
'
' Sample XML file
' ---------------
' <?xml version="1.0"?>
' <upgradeagent>
'  <title>Application Title</title>
'  <version>1.0.0</version>
'  <url>http://www.sageanalytic.com/UpgradeAgent/upgrade.msi</url>
' </upgradeagent>
'
'--------------------------------------------------------------------------------------------------------------------
' System Requirements:
'       Microsoft Visual Basic Runtime 6.0 or higher
'       Microsoft Internet Explorer 4.0 or higher
'       Microsoft XML Parser 2.0 or higher
'       Sage Analytic Upgrade Agent 1.0 or higher
'
' Make sure to reference "Microsoft XML, v3.0" or higher in your Visual Basic project
'
' Written by: Sage Analytic (http://www.sageanalytic.com)
'             04/23/2001
'
'====================================================================================================================

Declare Function GetCurrentProcessId Lib "kernel32" () As Long

Public Function CheckOnlineUpgrade(UpgradeSite As String) As Boolean

Dim objDOM As New MSXML2.DOMDocument
Dim UpgradeAvailable As Boolean
Dim UpgradeRequired As Boolean
Dim LocalFilename As String
Dim RemoteFilename As String
Dim VersionText As String
Dim CurrentMajor As Long
Dim CurrentMinor As Long
Dim CurrentRevision As Long
UpgradeAvailable = False
UpgradeRequired = False

On Error GoTo Finalize

objDOM.async = False
objDOM.Load UpgradeSite

If objDOM.parseError.reason <> "" Then
  GoTo Finalize
End If
    
VersionText = objDOM.selectSingleNode("//version").Text
RemoteFilename = objDOM.selectSingleNode("//url").Text
LocalFilename = Trim(App.Path) + "\" + Mid(RemoteFilename, InStrRev(RemoteFilename, "/") + 1)
CurrentMajor = CLng(Left(VersionText, InStr(1, VersionText, ".") - 1))
CurrentMinor = CLng(Mid(VersionText, InStr(1, VersionText, ".") + 1, InStrRev(VersionText, ".") - InStr(1, VersionText, ".") - 1))
CurrentRevision = CLng(Mid(VersionText, InStrRev(VersionText, ".") + 1))

If CurrentMajor > App.Major Then
  ' If Major has changed then an upgrade is mandatory
  UpgradeAvailable = True
  UpgradeRequired = True
ElseIf CurrentMajor = App.Major And CurrentMinor > App.Minor Then
  ' If only Minor has changed then an upgrade is optional
  UpgradeAvailable = True
  UpgradeRequired = False
ElseIf CurrentMajor = App.Major And CurrentMinor = App.Minor And CurrentRevision > App.Revision Then
  ' If only Revision has changed then an upgrade is optional and do not even prompt the user
  UpgradeAvailable = False
  UpgradeRequired = False
End If

If UpgradeRequired Then
  CheckOnlineUpgrade = True
  If MsgBox("An mandatory upgrade for this application is available. " + vbCrLf + _
      "The new version will be downloaded and installed automatically for you. " + vbCrLf + _
      "You will not be able to run this application until the upgrade has been performed.", _
      vbOKCancel + vbExclamation, "Application Version Check") = vbCancel Then
  UpgradeRequired = False
  End If
ElseIf UpgradeAvailable Then
  If MsgBox("An upgrade for this application is available. " + vbCrLf + vbCrLf + _
      "Do you wish to download and install the upgrade at this time?", vbYesNo + vbExclamation, _
      "Application Version Check") = vbYes Then
    CheckOnlineUpgrade = True
    UpgradeRequired = True
  End If
End If

If UpgradeRequired Then
  On Error Resume Next
  If Shell("""" + Trim(App.Path) + "\UpgradeAgent.exe"" /R " + RemoteFilename + _
      " /L """ + LocalFilename + """" + _
      " /A """ + App.Title + """" + _
      " /E """ + Trim(App.Path) + "\" + Trim(App.EXEName) + ".exe""" + _
      " /P " + CStr(GetCurrentProcessId()) + " /D", vbNormalFocus) = 0 Then
    MsgBox "ERROR: The upgrade agent could not be started.", vbCritical, "Application Version Check"
  End If
End If

Finalize:

Set objDOM = Nothing
End Function

'==============================================================================================

